@@ -216,7 +216,8 @@ struct mvapich_poll
 	} while (0);
 
 
-static void do_timings (mvapich_state_t *st, const char *fmt, ...);
+static void do_timings (mvapich_state_t *st, const char *fmt, ...)
+  __attribute__ ((format (printf, 2, 3)));
 void mvapich_thr_exit(mvapich_state_t *st);
 
 static int mvapich_requires_pids (mvapich_state_t *st)
@@ -283,7 +284,7 @@ char * vmsg (const char *msg, va_list ap)
 /*
  *  Forcibly kill job (with optional error message).
  */
-static int mvapich_terminate_job (mvapich_state_t *st, const char *msg, ...)
+static void mvapich_terminate_job (mvapich_state_t *st, const char *msg, ...)
 {
 	if (msg) {
 		va_list ap;
@@ -339,11 +340,11 @@ static void report_absent_tasks (mvapich_state_t *st, int check_do_poll)
 		hostlist_uniq (hosts);
 		int nranks = hostlist_count (tasks);
 		int nhosts = hostlist_count (hosts);
-		hostlist_ranged_string (tasks, 4096, r);
-		hostlist_ranged_string (hosts, 4096, h);
-		error ("mvapich: timeout: waiting on rank%s %s on host%s %s.\n",
-				nranks > 1 ? "s" : "", r,
-				nhosts > 1 ? "s" : "", h);
+		hostlist_ranged_string (tasks, sizeof(r), r);
+		hostlist_ranged_string (hosts, sizeof(h), h);
+		error ("mvapich: timeout: waiting on rank%s %s on host%s %s.",
+			nranks > 1 ? "s" : "", r,
+			nhosts > 1 ? "s" : "", h);
 	}
 
 	hostlist_destroy (hosts);
@@ -477,7 +478,7 @@ again:
 		if (mp->nfds == 0)
 			return (NULL);
 
-		mvapich_debug3 ("mvapich_poll_next (nfds=%d, timeout=%d)\n",
+		mvapich_debug3 ("mvapich_poll_next (nfds=%d, timeout=%d)",
 				mp->nfds, startup_timeout (st));
 		if ((rc = mvapich_poll_internal (mp)) < 0)
 			mvapich_terminate_job (st, "mvapich_poll_next: %m");
@@ -561,7 +562,8 @@ static int mvapich_write (struct mvapich_info *mvi, void * buf, size_t len)
 	n = write (mvi->fd, p, nleft);
 
 	if ((n < 0) && (errno != EAGAIN)) {
-		error ("mvapich: rank %d: write (%d/%ld): %m\n", mvi->rank, nleft, len);
+		error ("mvapich: rank %d: write (%zd/%zd): %m",
+		       mvi->rank, nleft, len);
 		return (-1);
 	}
 
@@ -588,7 +590,8 @@ static int mvapich_read (struct mvapich_info *mvi, void * buf, size_t len)
 	n = read (mvi->fd, p, nleft);
 
 	if ((n < 0) && (errno != EAGAIN)) {
-		error ("mvapich: rank %d: read (%d/%ld): %m\n", mvi->rank, nleft, len);
+		error ("mvapich: rank %d: read (%zd/%zd): %m",
+		       mvi->rank, nleft, len);
 		return (-1);
 	}
 
@@ -759,7 +762,7 @@ static void mvapich_bcast_hostids (mvapich_state_t *st)
 	/*
 	 *  Broadcast hostids
 	 */
-	mvapich_debug ("bcasting hostids\n");
+	mvapich_debug ("bcasting hostids");
 	mp = mvapich_poll_create (st);
 	while ((mvi = mvapich_poll_next (mp, 0))) {
 		if (mvapich_write (mvi, hostids, len) < 0)
@@ -776,10 +779,11 @@ static void mvapich_bcast_hostids (mvapich_state_t *st)
 	mvapich_poll_reset (mp);
 	while ((mvi = mvapich_poll_next (mp, 1))) {
 		int co = 1, rc;
-		mvapich_debug3 ("reading connect once value from rank %d fd=%d\n",
+		mvapich_debug3 ("reading connect once value from rank %d fd=%d",
 				mvi->rank, mvi->fd);
 		if ((rc = read (mvi->fd, &co, sizeof (int))) <= 0) {
-			mvapich_debug2 ("reading connect once value rc=%d: %m\n", rc);
+			mvapich_debug2 ("reading connect once value rc=%d: %m",
+					rc);
 			close (mvi->fd);
 			mvi->fd = -1;
 			st->connect_once = 0;
@@ -872,10 +876,10 @@ static int recv_common_value (mvapich_state_t *st, int *valp, int rank)
 {
 	int val;
 	if (mvapich_recv (st, &val, sizeof (int), rank) <= 0) {
-		error ("mvapich: recv_common_value: rank %d: %m\n", rank);
+		error ("mvapich: recv_common_value: rank %d: %m", rank);
 		return (-1);
 	}
-	mvapich_debug3 ("recv_common_value (rank=%d, val=%d)\n", rank, *valp);
+	mvapich_debug3 ("recv_common_value (rank=%d, val=%d)", rank, *valp);
 
 	/*
 	 *  If value is uninitialized, set it to current value,
@@ -908,7 +912,7 @@ static int process_pmgr_bcast (mvapich_state_t *st, int *rootp, int *sizep,
 	 *  Recv data from root
 	 */
 	*bufp = xmalloc (*sizep);
-	mvapich_debug3 ("PMGR_BCAST: recv from root\n");
+	mvapich_debug3 ("PMGR_BCAST: recv from root");
 	if (mvapich_recv (st, *bufp, *sizep, rank) < 0) {
 		error ("mvapich: PMGR_BCAST: Failed to recv from root: %m");
 		return (-1);
@@ -929,7 +933,7 @@ static int process_pmgr_gather (mvapich_state_t *st, int *rootp,
 	if (*bufp == NULL)
 		*bufp = xmalloc (*sizep * st->nprocs);
 
-	mvapich_debug3 ("PMGR_GATHER: recv from rank %d\n", rank);
+	mvapich_debug3 ("PMGR_GATHER: recv from rank %d", rank);
 	if (mvapich_recv(st, (*bufp) + (*sizep)*rank, *sizep, rank) < 0) {
 		error ("mvapich: PMGR_/GATHER: rank %d: recv: %m", rank);
 		return (-1);
@@ -971,7 +975,7 @@ static int process_pmgr_allgather (mvapich_state_t *st, int *sizep,
 	if (*bufp == NULL)
 		*bufp = xmalloc (*sizep * st->nprocs);
 
-	mvapich_debug3 ("PMGR_ALLGATHER: recv from rank %d\n", rank);
+	mvapich_debug3 ("PMGR_ALLGATHER: recv from rank %d", rank);
 	if (mvapich_recv (st, (*bufp) + *sizep*rank, *sizep, rank) < 0) {
 		error ("mvapich: PMGR_ALLGATHER: rank %d: %m", rank);
 		return (-1);
@@ -990,7 +994,7 @@ static int process_pmgr_alltoall (mvapich_state_t *st, int *sizep,
 
 	if (*bufp == NULL)
 		*bufp = xmalloc (*sizep * st->nprocs * st->nprocs);
-	mvapich_debug3 ("PMGR_ALLTOALL: recv from rank %d\n", rank);
+	mvapich_debug3 ("PMGR_ALLTOALL: recv from rank %d", rank);
 	if (mvapich_recv ( st,
 				*bufp + (*sizep * st->nprocs)*rank,
 				*sizep * st->nprocs, rank ) < 0) {
@@ -1017,7 +1021,7 @@ static int mvapich_process_op (mvapich_state_t *st,
 	}
 
 	opcode = *opcodep;
-	mvapich_debug3 ("rank %d: opcode=%d\n", mvi->rank, opcode);
+	mvapich_debug3 ("rank %d: opcode=%d", mvi->rank, opcode);
 
 	// read in additional data depending on current opcode
 
@@ -1174,9 +1178,9 @@ static int mvapich_processops (mvapich_state_t *st)
 	/* Until a 'CLOSE' or 'ABORT' message is seen, we continuously
 	 *  loop processing ops
 	 */
-	mvapich_debug ("Initiated PMGR processing\n");
+	mvapich_debug ("Initiated PMGR processing");
 	while (mvapich_pmgr_loop (st) != 1) {};
-	mvapich_debug ("Completed processing PMGR opcodes\n");
+	mvapich_debug ("Completed processing PMGR opcodes");
 
 	return (0);
 }
@@ -1221,6 +1225,7 @@ mvapich_print_abort_message (mvapich_state_t *st, int rank,
 	slurm_step_layout_t *sl = st->job->step_layout;
 	char *host;
 	char *msgstr;
+	char time_stamp[256];
 
 	if (!mvapich_abort_sends_rank (st)) {
 		info ("mvapich: Received ABORT message from an MPI process.");
@@ -1229,7 +1234,8 @@ mvapich_print_abort_message (mvapich_state_t *st, int rank,
 
 	if (msg && (msglen > 0)) {
 		/*
-		 *  Remove trailing newline if it exists (syslog will add newline)
+		 *  Remove trailing newline if it exists (syslog will
+		 *  add newline)
 		 */
 		if (msg [msglen - 1] == '\n')
 			msg [msglen - 1] = '\0';
@@ -1243,34 +1249,36 @@ mvapich_print_abort_message (mvapich_state_t *st, int rank,
 
 	host = slurm_step_layout_host_name (sl, rank);
 
+	LOG_TIMESTAMP(time_stamp);
 	if (dest >= 0) {
 		const char *dsthost = slurm_step_layout_host_name (sl, dest);
-
-		info ("mvapich: %M: ABORT from MPI rank %d [on %s] dest rank %d [on %s]",
-		      rank, host, dest, dsthost);
+		info ("mvapich: %s: ABORT from MPI rank %d "
+		      "[on %s] dest rank %d [on %s]",
+		      time_stamp, rank, host, dest, dsthost);
 
 		/*
 		 *  Log the abort event to syslog
-		 *   so that system administrators know about possible HW events.
+		 *  so that system administrators know about possible HW events.
 		 */
 		openlog ("srun", 0, LOG_USER);
 		syslog (LOG_WARNING,
-				"MVAPICH ABORT [jobid=%u.%u src=%d(%s) dst=%d(%s)]: %s",
-				st->job->jobid, st->job->stepid,
-				rank, host, dest, dsthost, msgstr);
+			"MVAPICH ABORT [jobid=%u.%u src=%d(%s) "
+			"dst=%d(%s)]: %s",
+			st->job->jobid, st->job->stepid,
+			rank, host, dest, dsthost, msgstr);
 		closelog();
 	} else {
-		info ("mvapich: %M: ABORT from MPI rank %d [on %s]",
-		      rank, host);
+		info ("mvapich: %s: ABORT from MPI rank %d [on %s]",
+		      time_stamp, rank, host);
 		/*
 		 *  Log the abort event to syslog
-		 *   so that system administrators know about possible HW events.
+		 *  so that system administrators know about possible HW events.
 		 */
 		openlog ("srun", 0, LOG_USER);
 		syslog (LOG_WARNING,
-				"MVAPICH ABORT [jobid=%u.%u src=%d(%s) dst=-1()]: %s",
-				st->job->jobid, st->job->stepid,
-				rank, host, msgstr);
+			"MVAPICH ABORT [jobid=%u.%u src=%d(%s) dst=-1()]: %s",
+			st->job->jobid, st->job->stepid,
+			rank, host, msgstr);
 		closelog();
 
 	}
@@ -1301,7 +1309,7 @@ static int mvapich_abort_timeout (void)
 
 static int mvapich_abort_accept (mvapich_state_t *st)
 {
-	slurm_addr addr;
+	slurm_addr_t addr;
 	int rc;
 	struct pollfd pfds[2];
 
@@ -1433,13 +1441,13 @@ static void do_timings (mvapich_state_t *st, const char *fmt, ...)
 
 	if (!initialized) {
 		if (gettimeofday (&initv, NULL) < 0)
-			error ("mvapich: do_timings(): gettimeofday(): %m\n");
+			error ("mvapich: do_timings(): gettimeofday(): %m");
 		initialized = 1;
 		return;
 	}
 
 	if (gettimeofday (&tv, NULL) < 0) {
-		error ("mvapich: do_timings(): gettimeofday(): %m\n");
+		error ("mvapich: do_timings(): gettimeofday(): %m");
 		return;
 	}
 
@@ -1449,8 +1457,8 @@ static void do_timings (mvapich_state_t *st, const char *fmt, ...)
 	msg = vmsg (fmt, ap);
 	va_end (ap);
 
-	info ("mvapich: %s took %d.%03d seconds", msg, result.tv_sec,
-			result.tv_usec/1000);
+	info ("mvapich: %s took %ld.%03ld seconds", msg,
+	      (long int)result.tv_sec, (long int)result.tv_usec/1000);
 
 	xfree (msg);
 
@@ -1470,8 +1478,10 @@ static int mvapich_read_item (struct mvapich_info *mvi, void *buf, size_t size)
 		if (errno == EAGAIN)
 			return (EAGAIN);
 		else {
-			error ("mvapich: %d: nread=%d, read (%d, %lx, size=%d, nleft=%d): %m",
-					mvi->rank, mvi->nread, mvi->fd, buf, size, nleft);
+			error ("mvapich: %d: nread=%d, read (%d, %zx, "
+			       "size=%zd, nleft=%zd): %m",
+			       mvi->rank, mvi->nread,
+			       mvi->fd, (size_t) buf, size, nleft);
 			return (-1);
 		}
 	}
@@ -1544,7 +1554,7 @@ again:
 
 	case MV_READ_HOSTID:
 		if (mvi->hostidlen != sizeof (int)) {
-			error ("mvapich: rank %d: unexpected hostidlen = %d\n",
+			error ("mvapich: rank %d: unexpected hostidlen = %d",
 					mvi->rank, mvi->hostidlen);
 			return (-1);
 		}
@@ -1624,7 +1634,7 @@ again:
  */
 static int mvapich_accept_new (mvapich_state_t *st)
 {
-	slurm_addr addr;
+	slurm_addr_t addr;
 	int fd;
 
 	/*
@@ -1656,7 +1666,7 @@ static int mvapich_accept_new (mvapich_state_t *st)
 		st->mvarray[st->nconnected]->fd = fd;
 		st->nconnected++;
 
-		mvapich_debug3 ("Got connection %d: fd=%d\n", st->nconnected, fd);
+		mvapich_debug3 ("Got connection %d: fd=%d", st->nconnected, fd);
 	}
 
 	return (0);
@@ -1714,7 +1724,7 @@ mvapich_initialize_connections (mvapich_state_t *st,
 		ncompleted = 0;
 
 		if (st->nconnected < st->nprocs)
-			mvapich_debug2 ("Waiting for connection %d/%d\n",
+			mvapich_debug2 ("Waiting for connection %d/%d",
 					st->nconnected + 1, st->nprocs);
 
 		for (i = 0; i < st->nconnected; i++) {
@@ -1732,7 +1742,7 @@ mvapich_initialize_connections (mvapich_state_t *st,
 		}
 
 		if (st->nconnected == st->nprocs && !printonce) {
-			mvapich_debug ("Got %d connections.\n", st->nprocs);
+			mvapich_debug ("Got %d connections.", st->nprocs);
 			do_timings (st, "Accept %d connection%s%s",
 					st->nprocs, st->nprocs == 1 ? "" : "s",
 					st->protocol_phase ? " (phase 2)" : "");
@@ -1746,7 +1756,7 @@ mvapich_initialize_connections (mvapich_state_t *st,
 			break; /* All done. */
 		}
 
-		mvapich_debug3 ("do_poll (nfds=%d)\n", nfds);
+		mvapich_debug3 ("do_poll (nfds=%d)", nfds);
 
 		while ((rc = poll (fds, nfds, startup_timeout (st))) < 0) {
 			if (errno == EINTR || errno == EAGAIN)
@@ -1759,7 +1769,7 @@ mvapich_initialize_connections (mvapich_state_t *st,
 			mvapich_terminate_job (st, NULL);
 		}
 
-		mvapich_debug3 ("poll (nfds=%d) = %d\n", nfds, rc);
+		mvapich_debug3 ("poll (nfds=%d) = %d", nfds, rc);
 
 		/*
 		 *  Stop other work if told to shut down
@@ -1853,7 +1863,7 @@ static int read_phase2_header (mvapich_state_t *st, struct mvapich_info *mvi)
 
 static int mvapich_handle_phase_two (mvapich_state_t *st)
 {
-	mvapich_debug ("protocol phase 0 complete. beginning phase 2.\n");
+	mvapich_debug ("protocol phase 0 complete. beginning phase 2.");
 
 	st->protocol_phase = 1;
 
@@ -2140,11 +2150,11 @@ extern mvapich_state_t *mvapich_thr_create(const mpi_plugin_client_info_t *job,
 
 	st = mvapich_state_create(job);
 	if (!st) {
-		error ("mvapich: Failed initialization\n");
+		error ("mvapich: Failed initialization");
 		return NULL;
 	}
 	if (process_environment (st) < 0) {
-		error ("mvapich: Failed to read environment settings\n");
+		error ("mvapich: Failed to read environment settings");
 		mvapich_state_destroy(st);
 		return NULL;
 	}
