@@ -1488,6 +1488,10 @@ extern int job_req_node_filter(struct job_record *job_ptr,
 			if ((detail_ptr->pn_min_cpus  > config_ptr->cpus)   ||
 			    ((detail_ptr->pn_min_memory & (~MEM_PER_CPU)) >
 			      config_ptr->real_memory) 			     ||
+			    ((detail_ptr->pn_min_memory & (MEM_PER_CPU)) &&
+			     ((detail_ptr->pn_min_memory & (~MEM_PER_CPU)) *
+			      detail_ptr->pn_min_cpus) >
+			     config_ptr->real_memory) 			     ||
 			    (detail_ptr->pn_min_tmp_disk >
 			     config_ptr->tmp_disk)) {
 				bit_clear(avail_bitmap, i);
@@ -1507,6 +1511,10 @@ extern int job_req_node_filter(struct job_record *job_ptr,
 			if ((detail_ptr->pn_min_cpus > node_ptr->cpus)     ||
 			    ((detail_ptr->pn_min_memory & (~MEM_PER_CPU)) >
 			      node_ptr->real_memory)                        ||
+			    ((detail_ptr->pn_min_memory & (MEM_PER_CPU)) &&
+			     ((detail_ptr->pn_min_memory & (~MEM_PER_CPU)) *
+			      detail_ptr->pn_min_cpus) >
+			     node_ptr->real_memory) 			   ||
 			    (detail_ptr->pn_min_tmp_disk >
 			     node_ptr->tmp_disk)) {
 				bit_clear(avail_bitmap, i);
